--- base/notifications.c.orig	2009-07-11 00:03:33.000000000 -0400
+++ base/notifications.c	2009-07-14 17:17:20.000000000 -0400
@@ -823,8 +823,36 @@
 		return FALSE;
 
 	/* skip this escalation if the state options don't match */
-	if(svc->current_state==STATE_OK && se->escalate_on_recovery==FALSE)
-		return FALSE;
+	if(svc->current_state==STATE_OK){
+		if (se->escalate_on_recovery==FALSE)
+			return FALSE;
+
+		/* If the escalation specifies one or more alert states (non-ok),  */
+		/* only allow the escalation if the non-ok state from the previous */
+		/* service state is listed in the escalation and if the escalation */
+		/* is NOT the first escalation (as that would mean there was no    */
+		/* non-ok escalation sent before the recovery escalation).         */
+
+		if ((se->escalate_on_unknown!=FALSE) || (se->escalate_on_warning!=FALSE) || (se->escalate_on_critical!=FALSE)){
+			
+			/* If the recovery is the first escalation, we had no previous */
+			/* non-ok escalation go out, so do not send a recovery         */
+			/* notification.                                               */
+        
+			if (svc->current_notification_number==se->first_notification)
+				return FALSE;
+
+			/* Only send out a recovery for the escalation if the previous     */
+			/* (non-ok) service state is a state also listed in the escalation */
+			if (temp_service->last_state==STATE_WARNING && se->escalate_on_warning==FALSE)
+				return FALSE;
+			if (temp_service->last_state==STATE_CRITICAL && se->escalate_on_critical==FALSE)
+				return FALSE;
+			if (temp_service->last_state==STATE_UNKNOWN && se->escalate_on_unknown==FALSE)
+				return FALSE;
+
+			} 
+		}
 	else if(svc->current_state==STATE_WARNING && se->escalate_on_warning==FALSE)
 		return FALSE;
 	else if(svc->current_state==STATE_UNKNOWN && se->escalate_on_unknown==FALSE)
